/*                                                                            */

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	#include <clib/alib_protos.h>
	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/graphics.h>
	#include <proto/intuition.h>
	#include <proto/datatypes.h>
	#include <datatypes/pictureclass.h>

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct animnode {
		struct Node n;
		Object *o;
		struct BitMap *bm;
		struct BitMapHeader *bmhd;
		};

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	char progress[] = "....................................................................................................";

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct List *load_anim (char *name,struct Screen *scr)

	{
	struct List *list = NULL;
	struct animnode *node;
	ULONG numpic = 1;
	ULONG n = 0;
	Object *o;
	BOOL ok = FALSE;

	if (o = NewDTObject (name,
			DTA_GroupID,GID_PICTURE,
			PDTA_Remap,TRUE,
			PDTA_Screen,scr,
			PDTA_GetNumPictures,&numpic,
			TAG_END))
		{
		Printf ("loading %s\n",name);
		Write (Output(),progress,numpic);
		Write (Output(),"\r*",2);

		if (list = AllocVec (sizeof(struct List),MEMF_CLEAR))
			{
			NewList (list);

			for (;;)
				{
				if (o)
					if (node = AllocVec (sizeof(struct animnode),MEMF_CLEAR))
						{
						DoMethod (o,DTM_PROCLAYOUT,NULL,TRUE);
						node->o = o;
						GetDTAttrs (o,
							PDTA_BitMap,&node->bm,
							PDTA_BitMapHeader,&node->bmhd,
							TAG_END);
						AddTail (list,&node->n);
						o = NULL;
						ok = TRUE;
						}

				n ++;
				if (n >= numpic)
					break;

				if (o)
					DisposeDTObject (o);

				Write (Output(),"*",1);

				o = NewDTObject (name,
					DTA_GroupID,GID_PICTURE,
					PDTA_Remap,TRUE,
					PDTA_Screen,scr,
					PDTA_WhichPicture,n,
					TAG_END);
				}
			}

		Write (Output(),"\n",1);

		if (o)
			DisposeDTObject (o);

		if (!ok)
			{
			FreeVec (list);
			list = NULL;
			}
		}

	return (list);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void free_anim (struct List *list)

	{
	struct animnode *node;

	if (list)
		{
		while (node = (struct animnode *) RemHead (list))
			{
			if (node->o)
				DisposeObject (node->o);

			FreeVec (node);
			}

		FreeVec (list);
		}
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct RDArgs *rdargs;
	struct {
		char *file;
		char *pubscreen;
		} args = {"Thomas:anim/bild1.gif",NULL};
	struct Screen *scr;
	struct List *anim;
	struct animnode *node;
	ULONG winw,winh;
	ULONG x,y,w,h;
	struct Window *win;
	struct IntuiMessage *imsg;
	BOOL cont;

	rdargs = ReadArgs ("FILE,PUBSCREEN/K",(LONG *)&args,NULL);
	if (!rdargs)
		{
		PrintFault (IoErr(),NULL);
		return (RETURN_ERROR);
		}

	if (scr = LockPubScreen (args.pubscreen))
		{
		if (anim = load_anim (args.file,scr))
			{
			node = (struct animnode *) anim->lh_Head;
			winw = node->bmhd->bmh_PageWidth;
			if (winw == 0)
				winw = node->bmhd->bmh_Width;
			winh = node->bmhd->bmh_PageHeight;
			if (winh == 0)
				winh = node->bmhd->bmh_Height;

			if (win = OpenWindowTags (NULL,
					WA_PubScreen,scr,
					WA_Flags,WFLG_BORDERLESS | WFLG_ACTIVATE | WFLG_NOCAREREFRESH,
					WA_IDCMP,IDCMP_VANILLAKEY | IDCMP_INTUITICKS,
					WA_Width,winw,
					WA_Height,winh,
					WA_Left,(scr->Width - winw) / 2,
					WA_Top,(scr->Height - winh) / 2,
					TAG_END))
				{
				cont = TRUE;
				do	{
					if (Wait ((1L << win->UserPort->mp_SigBit) | SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
						cont = FALSE;
					while (imsg = (struct IntuiMessage *) GetMsg (win->UserPort))
						{
						switch (imsg->Class)
							{
						case IDCMP_INTUITICKS:
							x = node->bmhd->bmh_Left;
							y = node->bmhd->bmh_Top;
							w = node->bmhd->bmh_Width;
							h = node->bmhd->bmh_Height;
							BltBitMapRastPort (node->bm,0,0,win->RPort,x,y,w,h,0xc0);
							node = (struct animnode *) node->n.ln_Succ;
							if (!node->n.ln_Succ)
								node = (struct animnode *)anim->lh_Head;
							break;
						case IDCMP_VANILLAKEY:
							if (imsg->Code == 0x1b)
								cont = FALSE;
							break;
							}
						ReplyMsg ((struct Message *)imsg);
						}
					}
				while (cont);

				CloseWindow (win);
				}

			free_anim (anim);
			}

		UnlockPubScreen (NULL,scr);
		}

	FreeArgs (rdargs);

	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/
